function [fitness_lookup,order] = convex_hull_search_greedydensity (in_pathway, outside_pathway, doplot)
%Bottom-up greedy BFS using distance from the point to the hull as a
%heuristic. To edit time limit change line 49.

%Setup the cell array for putting the fitness values in
fitness_lookup={[], [], []};

%Setup the values of the invariant variables for the cum_hyp_geom_dist
%function call
num_pathway=size(in_pathway,2);
num_total=num_pathway+size(outside_pathway,2);

%Calculate the starting convex hull (with everything)
%full_hull= convhull(in_pathway(1,:),in_pathway(2,:));

%set up minimum score seen so far
min_score = 1;
min_score_limit = 0;
prunings = 0;

%Setup the current hull and the pathway index logical lookup for the things
%contained in this hull
current_hull=[];
current_base=zeros(1,num_pathway);
seen_before=[];

%Setup the list of points from the pathway outside the hull
outside_hull = [1:num_pathway];

%Setup the flag that tells us if we need to continue the loop
cont=1;

%Setup the search tree - contains the list of branches waiting to be
%examined
%look_at=[];
look_at = containers.Map(1.0001, [1 0 1 1]);
look_at.remove(1.0001);
count=1;
%Now the slowest part of the code was the part where we checked through the
%fitness list in a for loop to see if we've seen a set before, so now treat
%the 1,0's in base as binary and make an array which is set to 1 when a thing is
%seen, then we can do direct lookups.
%seen_before = zeros (2^num_pathway, 1);
%Now changed so that it uses a list of hashed numbers rather than the
%binary lookup.

%time limit search
if num_pathway >= 10
    runtime_limit = (num_pathway^2) / 20;
else
    runtime_limit = 5;
end
tic;
starttime = toc;

disp('begun');

while cont==1 && ((toc - starttime) < runtime_limit)

    if (sum(current_base)>2)
        %Find all the points in the pathway outside the current hull
        %Matlab has a nicer way of doing this - using inpolygon I can do it for
        %all points simultaneously.
        inout=inpolygon(in_pathway(1,:),in_pathway(2,:), in_pathway(1,current_hull), in_pathway(2,current_hull));
        %So find gives the indices of the non-zero elements in an array, but I
        %want the zeros, so use not to switch them.
        outside_hull=find(not(inout));
   
        %Calculate the number inside the hull in the pathway
        num_sel_in_pway=sum(inout); 
   
   
        %check the non-pathway points to see if they're inside the hull and
        %increase the counter

        %Again do this the smarter way using inpolygon
        inout2=inpolygon(outside_pathway(1,:),outside_pathway(2,:), in_pathway(1,current_hull), in_pathway(2,current_hull));
        num_sel=sum(inout2)+num_sel_in_pway;
  
        %calculate the fitness of the current hull  
        %Let's try the built in function, which will at least get rid of
        %all the warnings, and hopefully might be quicker?
        fitness=hygecdf(num_sel_in_pway-1, num_total, num_sel, num_pathway, 'upper');
        score_k = num_sel;
        score_big_n = num_total;
        score_small_n = num_pathway;
        score_s = num_sel_in_pway;
        %min_score_limit = 0;
        min_score_limit = nchoosek(score_k + score_small_n + score_s, score_small_n) / nchoosek(score_big_n, score_small_n);
        if (fitness < min_score)
            min_score = fitness;
        end
        %The builtin function gives different values to my own code, whereas
        %the log hypergeometric function I found online gives the same results
        % Worked this out - this is cos the cdf in the built in function
        % assumes that you want the area from x+1 upwards ie. not including
        % the p(you pick x), whereas I want p(x or more).  
    
    
        %save the current hull along with fitness
        fitness_lookup{count,1} = current_hull;
        fitness_lookup{count,2}= fitness; 
        fitness_lookup{count,3} = outside_hull;
        fitness_lookup{count,4} = num_sel;
        fitness_lookup{count,5} = num_total;
        fitness_lookup{count,6} = num_pathway;
        fitness_lookup{count,7} = num_sel_in_pway;
        fitness_lookup{count,8} = in_pathway(:,current_hull);
        % fitness_lookup{count,4} = current_base;
        %seen_before(hash(current_base))=1;
        %seen_before=[];
        count=count+1;
    else
        outside_hull = find(not(current_base));
    end
    
    %setup base counter for how many branches we have waiting to be looked
    %at
    length_la=look_at.Count;
    %Keep count of the number of things added to look_at, so that we
    %can add without leaving empty slots.
    added=1;
    %Generate the list of branches which need looking at, by considering
    %the current set minus every point on the edge of the current hull
    for i=1:length(outside_hull)
        temp_base=current_base;
        temp_base(outside_hull(i))=1;
        if (sum(temp_base) > 2)
            temp_hull=convhull(in_pathway(1,temp_base==1),in_pathway(2,temp_base==1));
            temp_cum_sum_base=ones(1,size(temp_base,2));
            for i=1:size(temp_base,2)
                %Sum up the ones and multiple it by the current number to keep the
                %zeros
                temp_cum_sum_base(1,i)=sum(temp_base(1,1:i))*temp_base(1,i);
            end
            temp_hull2=[];
            for i=1:size(temp_hull)
                temp_hull2(i)=find(temp_cum_sum_base==temp_hull(i));
            end
            %CONVERT COORDINATES
            %maye done
            temp_base=inpolygon(in_pathway(1,:),in_pathway(2,:), in_pathway(1,temp_hull2), in_pathway(2,temp_hull2));
        end
        
        %Only add the new set to look at if it may contain something good
            
        %Only add to new set if it's not already been looked at.
        already_seen=0;
        h=hash(temp_base);
        % if (seen_before(h)==1)
        if (ismember(h,seen_before))
            already_seen=1;
        else
            if (min_score_limit < min_score)
                temp_points = [in_pathway(1, temp_base==1); in_pathway(2, temp_base==1)];
                current_points = [in_pathway(1, current_base==1); in_pathway(2, current_base==1)];
                distances_to_hull = pdist2(temp_points', current_points');
                r = rand( 1,'double');
                r = r/10000000000;
                if (length(current_points) > 2)
                    max_dist_to_hull = max(distances_to_hull);
                    max_dist_to_hull = max_dist_to_hull(1); 
                    look_at(max_dist_to_hull+r)=temp_base;
                else
                	look_at(r)=temp_base;
                end
                added=added+1;
            else
                prunings = prunings+1;
            end
            seen_before=[seen_before,h];
            %seen_before(h)=1;
        end
    end
    %stopping criteria
    if (look_at.Count == 0)
        cont=0;
    else    
        %Start looking at the first thing in the list
        K = look_at.keys();
        K = sort(cell2mat(K));
        best = K(1);
        current_base=look_at(best);
        look_at.remove(best);
        if (sum(current_base)>2)
            %Get the convex hull, but this will be in the temporary index system
            %from the current base, we then need to translate this back into the
            %real index system
            current_hull_temp=convhull(in_pathway(1,current_base==1),in_pathway(2,current_base==1));
            cum_sum_base=ones(1,size(current_base,2));
            for i=1:size(current_base,2)
                %Sum up the ones and multiple it by the current number to keep the
                %zeros
                cum_sum_base(1,i)=sum(current_base(1,1:i))*current_base(1,i);
            end
            %Now I'm using inpolygon I need to preserve the order of the
            %current_hull_temp, otherwise that lookup doesn't work as the right
            %polygon doesn't get made.
            %So I need the index where each number from current_hull_temp is
            %found in cum_sum_base.
            current_hull=[];
            for i=1:size(current_hull_temp)
                current_hull(i)=find(cum_sum_base==current_hull_temp(i));
            end
        end
    end
end

disp('ended');

[~, best_index]=min(cell2mat(fitness_lookup(:,2)));
[sorted,order] = sort(cell2mat(fitness_lookup(:,2)));
if (doplot==1)
    figure
    %recalculate the convex hull to get the points in the right order - NOT
    %NEEDED now as points already in right order.
%    hull=[in_pathway(1,fitness_lookup{best_index,1}); in_pathway(2,fitness_lookup{best_index,1})];
%    K=convhull(hull(1,:),hull(2,:));
    fill(current_hull(1,K),current_hull(2,K),'r');
    hold on
    alpha(0.5);
    plot(outside_pathway(1,:),outside_pathway(2,:),'xb');
    plot(in_pathway(1,:),in_pathway(2,:),'ob');
    hold off
end
    
    
    


    