function selected=greedy_find_set_descriptors_nointersect( pca_loadings, polygons, p_values)
%greedy_find_set_descriptors - Finds a set of descriptors which minimally
%overlap to describe a list of genes, using the pathways and GO-terms from
%CPDB output.
%   * pca_loadings - the pca_loadings for each gene, giving their position
%   in the pca-plot
%   * polygons - the best polygons found for each pathway
%   * p_values - the p-values for the original enrichment in each PC and
%   the p values of the best polygon. (3 by number of pathways)


num_sets = size(polygons, 2);

% gene and each set, and things to 1 when a gene
%is under the polygon in a particular set - 
    gene_set_matrix=zeros(num_sets, size(pca_loadings,1));
    for i=1:num_sets
        if ~isempty(polygons{i})
            gene_set_matrix(i,:)=inpolygon(pca_loadings(:,1),pca_loadings(:,2),polygons{i}(1,:),polygons{i}(2,:));
        end
    end

    
  
    %Now we want to know how many of the genes are selected in at least 1 set
    num_in_sets=sum(sum(gene_set_matrix)>0);
  %  fprintf('Percentage of genes covered by at least one polygon:');
  %  fprintf('\n %1.2f\n', num_in_sets/size(pca_loadings,1))

%Now to the greedy search. This is a pure greedy search, starts with the
%most significant set (at the top of the list), and then works down.
   % for i2=num_sets:-1:1
       selected=greedy_search(gene_set_matrix, 1, p_values, polygons);        
   % end
%Get the percentages of selected genes for each thing into one array
%     percent=zeros(num_sets,1);
%     all_ps=zeros(num_sets,1);
%     num_groups=zeros(num_sets,1);
%     for i4=1:num_sets
%         percent(i4)=selected(i4).percent;
%         all_ps(i4)=selected(i4).tot_p;
%         num_groups(i4)=selected(i4).num_groups;
%     end
%     %Quick hack so that it doesn't give you reults with only a single set
%     %(eg. a large general GO-term).
%     %combined_value=all_ps./num_groups.*(num_groups>=min_num_sets);
%     %combined_value(combined_value==0)=100000;
%     percent_comb=percent.*(num_groups>=min_num_sets);
%     [sorted,ind]=sort(percent_comb,'descend');
      
    end

    function sets_sel=greedy_search(gsm, seed, ps, polygons)
       %This function takes the gene_set_matrix (gsm) of any eligable sets,
       %ordered by their preferential usage (eg p-value) and a seed, which 
       %indicates where to start from. It then performs
       %a straightfoward greedy search. Finding groups where the overlap
       %with the current optimal set is less than overlap_allowed.
       genes_used=gsm(seed,:);
       sets_selected=zeros(size(gsm,1),1);
       sets_selected(seed)=1;
       %replace empty vectors in ps with 1, so these don't get selected
       for i=1:size(ps)
           if isempty(ps{i})
               ps{i}=1;
           end
       end
       psmat=cell2mat(ps);
       for i3=(seed+1):size(gsm,1)
           intersects_with_current_set=0;
           indices_current_set=find(sets_selected);
           for i4=1:sum(sets_selected)
               intersects_with_current_set=intersects_with_current_set + isintersect(polygons{i3}, polygons{indices_current_set(i4)});
           end
           if intersects_with_current_set==0
              genes_used=genes_used+gsm(i3,:); 
              sets_selected(i3)=1;
           end
       end 
    %   fprintf('Percentage of genes used')
       sets_sel.percent=sum(genes_used>0)/length(genes_used);
       if sets_sel.percent>1
           sets_sel.percent
           warning('percentage bigger than 100');
       end
       sets_sel.list=sets_selected;
    %   fprintf('Names');
    %   names(sets_selected==1)
       sets_sel.tot_p=sum(psmat(sets_selected==1));
       sets_sel.num_groups=sum(sets_selected);
    end
    


