function results = mygsea_v2(pathways, ranked_gene_list, PCvalues, lookupList)
%Gene set enrichment analysis
%This function takes;
%pathways read in from the cpdb output file for pathways
%a sorted ranked list of genes

%It returns a list of fdr corrected p-values for the significance of each
%pathway/geneset

%How many genes were supplied in the ranked list?
rkc=size(ranked_gene_list,1);

%parameters

randomisations=1000;

for i=1:size(pathways,1)

%First get the pathway gene names out of the pathway table
%     genes_pway=strsplit(char(pathways{i,4}),',');
%     
%     %setup a quick lookup list for which of the ranked genes are in the
%    %geneset
%    lookupList=zeros(rkc,1);
%    for j=1:size(genes_pway, 1)
%        lookupList(ranked_gene_list==genes_pway(j))=1;
%    end
   %Count how many genes this is
    overlap=sum(lookupList(:,i));
   
   %Now generate random lookup lists of this size
   randLookupIndices=ceil(rand(overlap,randomisations)*rkc);
   randLookupLists=zeros(rkc, randomisations);
   for k=1:randomisations
       randLookupLists(randLookupIndices(:,k),k)=1;
   end  
   %Next do the enrichment score calculations for all these lists together
   
    enrichAll=enrichingLookup([lookupList(:,i) randLookupLists], PCvalues);
   
%     %Check real enrichment of each gene set
%     enrichmentScores(i)=enriching(genes_pway,ranked_gene_list, PCvalues);
%     %Now check permuations of the ranked gene list for that gene set
%     for j=1:randomisations
%         ESrand(j)=enriching(genes_pway, ranked_gene_list(randperm(rkc)), PCvalues);
%     end

    ESrand=enrichAll(2:end);
    enrichmentScores(i)=enrichAll(1);
    
    p(i)=(sum(ESrand>=enrichmentScores(i))/randomisations);
end

%fdr correct the results
results=mafdr(p);
end

function MaxES = enrichingLookup(lookupLists, values)
%This function calculates the enrichment score for a set of lookuplists at
%the same time (eg real and all randomisations for a single pathway
   es(size(lookupLists,2),1)=0;
   rkc=size(lookupLists,1);
   gsc=sum(lookupLists(1,:));
   %Calculate the absolute value of the sum of the values in each lookup
   %list - I tried doing this the matrix way, but it was too memory heavy
   s=zeros(size(lookupLists,2),1);
   for i=1:size(lookupLists,2)
    s(i) = sum(abs(values(lookupLists(:,i)==1)));
   end
   
   %decrease is set based on the size of the gene set
   decrease=-1/(rkc-gsc);
      %go through the ranked list of genes
    for i2=2:rkc+1
        %First copy old score across from before
        es(:,i2)=es(:,i2-1);
       %Do decrease for the places where the current gene is not in the
       %lookuplist : note decrease is negative so it gets added.
       es(:,i2)=es(:,i2)+(repmat(decrease,1,size(lookupLists,2)).*(lookupLists(250,:)==0))';
        %then do increase
      es(:,i2)=es(:,i2)+(repmat(abs(values(i2-1)),size(s,1),1)./s).*lookupLists(i2-1);
   end
   MaxES=max(es');
end

